function dstat = compute_dstat(tiwt,gamma,cflag);

% compute_dstat --  compute shift dependent detection statistic
%                   for signal in the presence of 1/f^gamma noise
%                
% Usage
%  dstat = compute_dstat(tiwt,gamma,cflag);
%
% Inputs
%  tiwt		translation invariant wt data structure
%  gamma	1/f^gamma parameter
%  cflag        1: use coarse approximation in detection statistic
%               0: only use detail approx in det. stat.
%               default = 0;
%
% Outputs
%  dstat	vector of detection statistics corresponding
%		to different tiwt shifts
%
% Uses
%   packet, mybin2dec, binary_inc
%
% References
%  G. Wornell, Signal Processing with Fractals, 1996, p. 79-84.
%   note: Wornell computes detection stat for only one arbitrary shift.

	if ~exist('cflag')
          cflag = 0;
   	end
         


	[n,M] = size(tiwt);
	M = M-1;

%  make up a lookup table into the tiwt structure
%  The indexing algorithm is taken from the Wavelab function
%  TI2Stat.m written by Shaobing Chen.
%  At the same time compute energy of each block of the tiwt.

       totalshifts = 2^M;
       dstat = zeros(totalshifts,1);
       lookup = zeros(M,totalshifts);
       energydist = zeros(M,totalshifts);

       index = 1;
       for m = 1:M
          nb = 2^m;
          index = [(index + nb/2);index];
          index = index(:)';
          lookup(m,1:nb) = reverse(index);

          for b = 0:nb-1
            energydist(m,b+1) = sum(tiwt(m*n + packet(m,b,n)).^2);
          end
       end

       if cflag
          for b = 0:nb-1
              energydist(m,b+1) = energydist(m,nb-b) + sum(tiwt(packet(m,b,n)).^2);
          end
       end

%  compute the detection statistic
	dstat = zeros(totalshifts,1);
	binary_num = zeros(1,M);

 	for k = 1:totalshifts
          for m = 1:M
            bin_index = binary_num((M-m+1):M);
            bin_index = mybin2dec(bin_index)+1;
            dstat(k) =  dstat(k) + ...
                            energydist(m,lookup(m,bin_index))*2^(-gamma*m);
          end
          binary_num = binary_inc(binary_num);
        end

    
       



% Copyright (c) 1998 Thomas T. Liu
%
% Note this function works in conjunction with
% routines and data structures from Wavelab version .701 which is copyrighted
% material from D.L. Donoho's group at Stanford University. 
